@@ -243,13 +243,15 @@ class TestDateFields(ModelTestCase):
                  .tuples())
 
         row, = query
-        if IS_SQLITE or IS_MYSQL or IS_CRDB:
+        if IS_SQLITE or IS_MYSQL:
             self.assertEqual(row,
                              (2011, 1, 2, 11, 12, 13, 2012, 2, 3, 3, 13, 37))
         else:
-            self.assertEqual(row, (
-                2011., 1., 2., 11., 12., 13.054321, 2012., 2., 3., 3., 13.,
-                37.))
+            self.assertTrue(row in [
+                (2011., 1., 2., 11., 12., 13.054321, 2012., 2., 3., 3., 13.,
+                 37.),
+                (D('2011'), D('1'), D('2'), D('11'), D('12'), D('13.054321'),
+                 D('2012'), D('2'), D('3'), D('3'), D('13'), D('37'))])
 
     def test_truncate_date(self):
         dm = DateModel.create(
@@ -561,6 +563,31 @@ class TestIPField(ModelTestCase):
 class TestBitFields(ModelTestCase):
     requires = [Bits]
 
+    def test_bit_field_update(self):
+        def assertFlags(expected):
+            query = Bits.select().order_by(Bits.id)
+            self.assertEqual([b.flags for b in query], expected)
+
+        # Bits - flags (1=sticky, 2=favorite, 4=minimized)
+        for i in range(1, 5):
+            Bits.create(flags=i)
+
+        Bits.update(flags=Bits.flags & ~2).execute()
+        assertFlags([1, 0, 1, 4])
+
+        Bits.update(flags=Bits.flags | 2).execute()
+        assertFlags([3, 2, 3, 6])
+
+        Bits.update(flags=Bits.is_favorite.clear()).execute()
+        assertFlags([1, 0, 1, 4])
+
+        Bits.update(flags=Bits.is_favorite.set()).execute()
+        assertFlags([3, 2, 3, 6])
+
+        # Clear multiple bits in one operation.
+        Bits.update(flags=Bits.flags & ~(1 | 4)).execute()
+        assertFlags([2, 2, 2, 2])
+
     def test_bit_field_auto_flag(self):
         class Bits2(TestModel):
             flags = BitField()
@@ -620,6 +647,13 @@ class TestBitFields(ModelTestCase):
         query = Bits.select().where((Bits.flags & 1) == 1).order_by(Bits.id)
         self.assertEqual([x.id for x in query], [b1.id, b3.id])
 
+        # Test combining multiple bit expressions.
+        query = Bits.select().where(Bits.is_sticky & Bits.is_favorite)
+        self.assertEqual([x.id for x in query], [b3.id])
+
+        query = Bits.select().where(Bits.is_sticky & ~Bits.is_favorite)
+        self.assertEqual([x.id for x in query], [b1.id])
+
     def test_bigbit_field_instance_data(self):
         b = Bits()
         values_to_set = (1, 11, 63, 31, 55, 48, 100, 99)
@@ -644,6 +678,35 @@ class TestBitFields(ModelTestCase):
         b.data.clear_bit(0)
         self.assertFalse(b.data.is_set(0))
 
+        # Out-of-bounds returns False and does not extend data.
+        self.assertFalse(b.data.is_set(1000))
+        self.assertTrue(len(b.data), 1)
+
+    def test_bigbit_item_methods(self):
+        b = Bits()
+        idxs = [0, 1, 4, 7, 8, 15, 16, 31, 32, 63]
+        for i in idxs:
+            b.data[i] = True
+        for i in range(64):
+            self.assertEqual(b.data[i], i in idxs)
+
+        data = list(b.data)
+        self.assertEqual(data, [1 if i in idxs else 0 for i in range(64)])
+
+        for i in range(64):
+            del b.data[i]
+        self.assertEqual(len(b.data), 8)
+        self.assertEqual(b.data._buffer, b'\x00' * 8)
+
+    def test_bigbit_set_clear(self):
+        b = Bits()
+        b.data = b'\x01'
+        for i in range(8):
+            self.assertEqual(b.data[i], i == 0)
+
+        b.data.clear()
+        self.assertEqual(len(b.data), 0)
+
     def test_bigbit_field(self):
         b = Bits.create()
         b.data.set_bit(1)
@@ -658,6 +721,60 @@ class TestBitFields(ModelTestCase):
             else:
                 self.assertFalse(b_db.data.is_set(x))
 
+    def test_bigbit_field_bitwise(self):
+        b1 = Bits(data=b'\x11')
+        b2 = Bits(data=b'\x12')
+        b3 = Bits(data=b'\x99')
+        self.assertEqual(b1.data & b2.data, b'\x10')
+        self.assertEqual(b1.data | b2.data, b'\x13')
+        self.assertEqual(b1.data ^ b2.data, b'\x03')
+        self.assertEqual(b1.data & b3.data, b'\x11')
+        self.assertEqual(b1.data | b3.data, b'\x99')
+        self.assertEqual(b1.data ^ b3.data, b'\x88')
+
+        b1.data &= b2.data
+        self.assertEqual(b1.data._buffer, b'\x10')
+
+        b1.data |= b2.data
+        self.assertEqual(b1.data._buffer, b'\x12')
+
+        b1.data ^= b3.data
+        self.assertEqual(b1.data._buffer, b'\x8b')
+
+        b1.data = b'\x11'
+        self.assertEqual(b1.data & b'\xff\xff', b'\x11\x00')
+        self.assertEqual(b1.data | b'\xff\xff', b'\xff\xff')
+        self.assertEqual(b1.data ^ b'\xff\xff', b'\xee\xff')
+
+        b1.data = b'\x11\x11'
+        self.assertEqual(b1.data & b'\xff', b'\x11\x00')
+        self.assertEqual(b1.data | b'\xff', b'\xff\x11')
+        self.assertEqual(b1.data ^ b'\xff', b'\xee\x11')
+
+    def test_bigbit_field_bulk_create(self):
+        b1, b2, b3 = Bits(), Bits(), Bits()
+        b1.data.set_bit(1)
+        b2.data.set_bit(2)
+        b3.data.set_bit(3)
+        Bits.bulk_create([b1, b2, b3])
+        self.assertEqual(len(Bits), 3)
+        for b in Bits.select():
+            self.assertEqual(sum(1 if b.data.is_set(i) else 0
+                                 for i in (1, 2, 3)), 1)
+
+    def test_bigbit_field_bulk_update(self):
+        b1, b2, b3 = Bits.create(), Bits.create(), Bits.create()
+
+        b1.data.set_bit(11)
+        b2.data.set_bit(12)
+        b3.data.set_bit(13)
+        Bits.bulk_update([b1, b2, b3], fields=[Bits.data])
+
+        mapping = {b1.id: 11, b2.id: 12, b3.id: 13}
+        for b in Bits.select():
+            bit = mapping[b.id]
+            self.assertTrue(b.data.is_set(bit))
+
 
 class BlobModel(TestModel):
     data = BlobField()
@@ -1255,6 +1372,19 @@ class TestForeignKeyLazyLoad(ModelTestCase):
             b = NQ.create(name='b')
             bi = NQItem.create(nq=b, nq_lazy=b)
 
+    def test_doesnotexist_lazy_load(self):
+        n = NQ.create(name='n1')
+        i = NQItem.create(nq=n, nq_null=n, nq_lazy=n, nq_lazy_null=n)
+
+        i_db = NQItem.select(NQItem.id).where(NQItem.nq == n).get()
+        with self.assertQueryCount(0):
+            # Only raise DoesNotExist for non-nullable *and* lazy-load=True.
+            # Otherwise we just return None.
+            self.assertRaises(NQ.DoesNotExist, lambda: i_db.nq)
+            self.assertTrue(i_db.nq_null is None)
+            self.assertTrue(i_db.nq_lazy is None)
+            self.assertTrue(i_db.nq_lazy_null is None)
+
     def test_foreign_key_lazy_load(self):
         a1, a2, a3, a4 = (NQ.select()
                           .where(NQ.name.startswith('a'))
@@ -1281,6 +1411,17 @@ class TestForeignKeyLazyLoad(ModelTestCase):
         with self.assertQueryCount(1):
             self.assertEqual(bi.nq.id, b.id)
 
+    def test_fk_lazy_load_related_instance(self):
+        nq = NQ(name='b1')
+        nqi = NQItem(nq=nq, nq_null=nq, nq_lazy=nq, nq_lazy_null=nq)
+        nq.save()
+        nqi.save()
+
+        with self.assertQueryCount(1):
+            nqi_db = NQItem.get(NQItem.id == nqi.id)
+            self.assertEqual(nqi_db.nq_lazy, nq.id)
+            self.assertEqual(nqi_db.nq_lazy_null, nq.id)
+
     def test_fk_lazy_select_related(self):
         NA, NB, NC, ND = [NQ.alias(a) for a in ('na', 'nb', 'nc', 'nd')]
         LO = JOIN.LEFT_OUTER
