@@ -7,6 +7,7 @@ from types import MethodType
 
 from peewee import *
 from playhouse.postgres_ext import *
+from playhouse.reflection import Introspector
 
 from .base import BaseTestCase
 from .base import DatabaseTestCase
@@ -69,7 +70,7 @@ try:
 
     class JData(TestModel):
         d1 = BinaryJSONField()
-        d2 = BinaryJSONField()
+        d2 = BinaryJSONField(index=False)
 except:
     BJson = JData = None
     JsonModel = JsonModelNull = None
@@ -372,6 +373,9 @@ class TestArrayField(ModelTestCase):
         assertAM(T.contains('omega', 'delta'))
         assertAM(T.contains('汉语'), am5)
         assertAM(T.contains('alpha', 'delta'), am)
+        assertAM(T.contained_by('alpha', 'beta', 'delta'), am2, am3)
+        assertAM(T.contained_by('alpha', 'beta', 'gamma', 'delta'),
+                 am, am2, am3)
 
         # Check for any.
         assertAM(T.contains_any('beta'), am, am2)
@@ -609,6 +613,18 @@ class TestBinaryJsonField(BaseBinaryJsonFieldTestCase, ModelTestCase):
         assertData(['k1', 'kx', 'ky', 'k3'], {'k2': 'v2', 'k4': [0, 1, 2]})
         assertData(['k4', 'k3'], {'k1': 'v1', 'k2': 'v2'})
 
+    @skip_unless(pg10(), 'jsonb remove support requires pg >= 10')
+    def test_json_contains_in_list(self):
+        m1 = self.M.create(data=[{'k1': 'v1', 'k2': 'v2'}, {'a1': 'b1'}])
+        m2 = self.M.create(data=[{'k3': 'v3'}, {'k4': 'v4'}])
+        m3 = self.M.create(data=[{'k5': 'v5', 'k6': 'v6'}, {'k1': 'v1'}])
+
+        query = (self.M
+                 .select()
+                 .where(self.M.data.contains([{'k1': 'v1'}]))
+                 .order_by(self.M.id))
+        self.assertEqual([m.id for m in query], [m1.id, m3.id])
+
     def test_integer_index_weirdness(self):
         self._create_test_data()
 
@@ -655,6 +671,17 @@ class TestJsonFieldRegressions(ModelTestCase):
         self.assertEqual(obj.data, {
             'k1': {'x2': 'y2'}, 'k2': 'v2-x', 'k3': 'v3', 'k4': 'v4'})
 
+    def test_introspect_bjson_field(self):
+        introspector = Introspector.from_database(self.database)
+        models = introspector.generate_models(table_names=['j_data'])
+        JD = models['j_data']
+        self.assertEqual(JD._meta.sorted_field_names, ['id', 'd1', 'd2'])
+        self.assertTrue(isinstance(JD.d1, BinaryJSONField))
+        self.assertTrue(isinstance(JD.d2, BinaryJSONField))
+        self.assertTrue(JD.d1.index)
+        self.assertEqual(JD.d1.index_type, 'GIN')
+        self.assertFalse(JD.d2.index)
+
 
 class TestIntervalField(ModelTestCase):
     database = db
@@ -788,6 +815,16 @@ class TestServerSide(ModelTestCase):
         ss_query = ServerSide(query.where(SQL('1 = 0')))
         self.assertEqual(list(ss_query), [])
 
+    def test_lower_level_apis(self):
+        query = Register.select(Register.value).order_by(Register.value)
+        ssq = ServerSideQuery(query, array_size=10)
+        curs_wrapper = ssq._execute(self.database)
+        curs = curs_wrapper.cursor
+        self.assertTrue(isinstance(curs, FetchManyCursor))
+        self.assertEqual(curs.fetchone(), (0,))
+        self.assertEqual(curs.fetchone(), (1,))
+        curs.close()
+
 
 class KX(TestModel):
     key = CharField(unique=True)
@@ -806,13 +843,11 @@ class TestAutocommitIntegration(ModelTestCase):
         # Force an integrity error, then verify that the current
         # transaction has been aborted.
         self.assertRaises(IntegrityError, KX.create, key='k1', value=10)
-        self.assertRaises(InternalError, KX.get, key='k1')
 
     def test_autocommit_default(self):
         kx2 = KX.create(key='k2', value=2)  # Will be committed.
         self.assertTrue(kx2.id > 0)
         self.force_integrity_error()
-        self.database.rollback()
 
         self.assertEqual(KX.select().count(), 2)
         self.assertEqual([(kx.key, kx.value)
@@ -821,6 +856,7 @@ class TestAutocommitIntegration(ModelTestCase):
 
     def test_autocommit_disabled(self):
         with self.database.manual_commit():
+            self.database.begin()
             kx2 = KX.create(key='k2', value=2)  # Not committed.
             self.assertTrue(kx2.id > 0)  # Yes, we have a primary key.
             self.force_integrity_error()
@@ -936,3 +972,26 @@ class TestPostgresLateralJoin(ModelTestCase):
             ('b', 'b4'),
             ('b', 'b7'),
             ('c', None)])
+
+    @requires_models(User, Tweet)
+    def test_lateral_helper(self):
+        self.create_data()
+
+        subq = (Tweet
+                .select(Tweet.content, Tweet.timestamp)
+                .where(Tweet.user == User.id)
+                .order_by(Tweet.timestamp.desc())
+                .limit(2)
+                .lateral())
+
+        query = (User
+                 .select(User, subq.c.content)
+                 .join(subq, on=True)
+                 .order_by(subq.c.timestamp.desc(nulls='last')))
+        with self.assertQueryCount(1):
+            results = [(u.username, u.tweet.content) for u in query]
+            self.assertEqual(results, [
+                ('a', 'a10'),
+                ('b', 'b7'),
+                ('b', 'b4'),
+                ('a', 'a2')])
